import pandas as pd
import numpy as np
import matplotlib.pylab as plt
from sklearn.model_selection import StratifiedKFold, cross_val_score
from sklearn.metrics import roc_auc_score
from scipy import interp
from sklearn.metrics import roc_curve, auc
from sklearn.linear_model import LogisticRegression, LogisticRegressionCV
from sklearn import svm
from xgboost import XGBClassifier
from sklearn.ensemble import RandomForestClassifier
from catboost import CatBoostClassifier
from dhcdatalearn.model_evaluation import EvaModelFusion
import warnings
warnings.filterwarnings('ignore')


def calres(clf, col, names):
    eva = EvaModelFusion()
    eva.train(X[col], y)
    sampling_methods = [clf]
    foldres = cross_val_score(clf, X[col], y, scoring="roc_auc", cv=cv)
    foldres = np.array(foldres)
    return eva.evalu_models(names, sampling_methods, X[col], y), foldres, foldres.mean()

def features_plot(clf, feature, lr=False):
    if lr:
        importance = clf.coef_[0]
    else:
        importance = clf.feature_importances_
    feature_importance = pd.Series(importance, index=feature).sort_values(ascending=False)
    sorted_names = feature_importance.index.values
    sorted_scores = feature_importance.values
    y_pos = np.arange(len(sorted_names))
    plt.figure(figsize=(20, 20), dpi=50)
    fig, ax = plt.subplots(figsize=(15,15), dpi=50)
    ax.barh(y_pos, sorted_scores, height=0.7, align='center', color='green', tick_label=sorted_names)
    ax.set_yticks(y_pos)
    plt.yticks(fontsize=20)
    ax.set_xlabel('Feature Score')
    ax.set_ylabel('Features')
    ax.invert_yaxis()
    ax.set_title('Feature Importance')
    for score, pos in zip(sorted_scores, y_pos):
        ax.text(score + 0.005, pos, '%.5f' % score, ha='center', va='center', fontsize=16)


path = 'D:/DHC/DHC栗翊超交接/项目/10 华西脑出血/code/Data/'
df = pd.read_csv(path + 'NCX_Recovery.csv',encoding='gbk')
y_true = df['Recovery state']
tt = []
for i in y_true:
    if i == 0:
        tt.append(1)
    else:
        tt.append(0)
y_true = pd.DataFrame(tt)
df['Recovery'] = y_true
features = ['Temperature',
             'AMC',
             'Initial hematoma volume',
             'Baseline heart rate',
             'D-dimer',
             'WBC',
             'Chlorine',
             'Blood glucose',
             'GFR',
             'Cr',
             'ANC',
             'Age',
             'Intraventricular hemorrhage',
             'Coagulation disorders',
             'Location of the hematoma',
             'GCS']
target = 'Recovery'
X = df[features]
y = df[target]
cv = StratifiedKFold(n_splits=5, shuffle=True, random_state=0)

# LRCV
lrcv_clf = LogisticRegressionCV(class_weight='balanced', penalty='l2', solver='liblinear')
lrcv_col = ['Temperature',
            'AMC',
            'D-dimer',
            'WBC',
            'Blood glucose',
            'ANC',
            'Age', 'Intraventricular hemorrhage',
       'Coagulation disorders', 'Location of the hematoma', 'GCS']
names = ['LogisticRegressionCV']
lrcvallres, lrcvfoldres, lrcvfoldmean = calres(lrcv_clf, lrcv_col, names)
lrcv_clf.fit(X[lrcv_col], y)
features_plot(lrcv_clf, lrcv_col, lr=True)

# LR
lr_clf = LogisticRegression(class_weight='balanced')
lr_col = ['Temperature', 'AMC', 'D-dimer', 'Chlorine', 'Blood glucose', 'ANC',
          'Age', 'Intraventricular hemorrhage',
       'Coagulation disorders', 'Location of the hematoma', 'GCS']
names = ['LogisticRegression']
lrallres, lrfoldres, lrfoldmean = calres(lr_clf, lr_col, names)
lr_clf.fit(X[lr_col], y)
features_plot(lr_clf, lr_col, lr=True)

# XGBoost
xgb_clf = XGBClassifier(
                    objective='binary:logistic'
                    ,learning_rate=0.01
                    ,n_estimators=161
                    ,max_depth=4
                    ,min_child_weight=5
                    ,gamma=13
                    ,subsample=0.4
                    ,colsample_bytree=1
                    ,scale_pos_weight=2.792929292929293
                    ,reg_alpha=0.59
                    ,reg_lambda=1
                    ,n_jobs=-1
                    ,seed=2021)
xgb_col = ['Initial hematoma volume', 'D-dimer', 'WBC', 'Chlorine',
       'Blood glucose', 'GFR', 'Cr',
           'Age',
       'Intraventricular hemorrhage', 'Location of the hematoma', 'GCS']
names = ['XGBoost']
xgballres, xgbfoldres, xgbfoldmean = calres(xgb_clf, xgb_col, names)
xgb_clf.fit(X[xgb_col], y)
features_plot(xgb_clf, xgb_col)

# RF
rf_clf = RandomForestClassifier(n_estimators=42, max_depth=3, class_weight='balanced',random_state=0,
                               min_samples_leaf=30, min_samples_split=2,bootstrap=True,criterion='gini',n_jobs=-1)
rf_col = ['Initial hematoma volume', 'D-dimer', 'Blood glucose',
       'Intraventricular hemorrhage', 'Location of the hematoma', 'GCS']
names = ['RandomForest']
rfallres, rffoldres, rffoldmean = calres(rf_clf, rf_col, names)
rf_clf.fit(X[rf_col], y)
features_plot(rf_clf, rf_col)

# svm
svm_col = ['Temperature',
             'AMC',
             'Initial hematoma volume',
             'Baseline heart rate',
             'D-dimer',
             'WBC',
             'Chlorine',
             'Blood glucose',
             'GFR',
             'Cr',
             'ANC',
             'Age',
             'Intraventricular hemorrhage',
             'Coagulation disorders',
             'Location of the hematoma',
             'GCS']
svm_clf = svm.SVC(C = 10, gamma = 0.00002,kernel='rbf', probability=True, class_weight='balanced')
names = ['svm']
svmallres, svmfoldres, svmfoldmean = calres(svm_clf, svm_col, names)

# CatBoost
cat_col = ['Blood glucose', 'D-dimer', 'GCS']

bestparam = {'depth': 3, 'iterations': 140, 'learning_rate': 0.05, 'l2_leaf_reg': 1, 
              'scale_pos_weight': 0.64, 'thread_count': 4, 'random_seed': 0, 'silent':True}
cat_clf = CatBoostClassifier(**bestparam)
names = ['catboost']
catallres, catfoldres, catfoldmean = calres(cat_clf, cat_col, names)
cat_clf.fit(X[cat_col], y)
features_plot(cat_clf, cat_col)

# ROC
def plot_rocs(names, clfs, dpin=100, save=False):
    plt.figure(figsize=(6, 6), dpi=dpin)
    for (name, clf) in zip(names, clfs):
        print(name)

        tprs = []
        aucs = []
        mean_fpr = np.linspace(0, 1, 100)
        i = 0
        for train, test in cv.split(X, y.values):
            if name != 'ICH score':
                probas_ = clf[0].fit(X[clf[1]].values[train], y[train]).predict_proba(X[clf[1]].values[test])
                fpr, tpr, thresholds = roc_curve(y[test], probas_[:, 1])
            else:
                fpr, tpr, thresholds = roc_curve(y[test], y_scores[test])
            tprs.append(interp(mean_fpr, fpr, tpr))
            tprs[-1][0] = 0.0
            roc_auc = auc(fpr, tpr)
            aucs.append(roc_auc)

            i += 1
        plt.plot([0, 1], [0, 1], linestyle='--', lw=2, color='r', alpha=.8)

        mean_tpr = np.mean(tprs, axis=0)
        mean_tpr[-1] = 1.0
        mean_auc = auc(mean_fpr, mean_tpr)
        std_auc = np.std(aucs)
        tname = name
        
        if name != 'ICH score':
            plt.plot(mean_fpr, mean_tpr,
                     label=tname,
                     lw=2, alpha=.8)
            
        else:
            plt.plot(mean_fpr, mean_tpr,
                     label=tname,
                     lw=2, color='black', alpha=.8)
        plt.xlim([-0.05, 1.05])
        plt.ylim([-0.05, 1.05])
        plt.xticks(fontsize=11)
        plt.yticks(fontsize=11)
        plt.xlabel('False Positive Rate', fontsize=12)
        plt.ylabel('True Positive Rate', fontsize=12)
        plt.title('ROC Curve', fontsize=12)
        plt.legend(fontsize=4)
        plt.legend(loc="lower right")
        if save:
            plt.savefig('ROC_Recovery.tiff')


ich = pd.read_excel("ichdata.xlsx")
y_scores = ich['ICH score']

names = ["ICH score",
         "LR",
         "LRCV",
         "SVM",
         "RF",
         "XGBoost",
         "CatBoost"
        ]

clfs = [(y_scores, y_scores),
        (lr_clf, lr_col),
        (lrcv_clf, lrcv_col),
        (svm_clf, svm_col),
        (rf_clf, rf_col),
        (xgb_clf, xgb_col),
        (cat_clf, cat_col)
        ]

plot_rocs(names, clfs, dpin=600, save=True)